/*       
  ****************

  -= RC5=-   

  ----------------    
  StartCd.narod.ru   
  ----------------   
   
    
   
  *****************
*/

// F_CPU = 1000000
// UART baud rate = 4800, 8bit data, 2stop bit.


/*
	|   __|
	|__|  |
	|     |
1  = 1.778 mSec
	
    14  = 24,892 mSec
      64  x 1.778 mS = 113.792 mSec
 -    50  = 88.889 mSec	

        1     1    1  |  0     1     1     0     0  |  1
	|   __|	  __|   __|__   |   __|	  __|__   |__   |   __|	  __|   __|	  __|   __|	  __|
	|__|  |__|  |__|  |  |__|__|  |__|  |  |__|  |__|__|  |__|  |__|  |__|  |__|  |__|  |____
	|     |	    |     |	    |     |	    |     |	    |     |	    |     |	    |     |	    |
       s1    s2  ..   5               |  6 
            |     |     |     |     |     |     |     |     |     |     |     |      

   .	
      e()
   ,    
  .

  ,   .
 ,         
     ,     .
*/


#include <avr\io.h> 			
#include <avr\interrupt.h>		
#include <stdlib.h>				

// RC5_T-    1,778mSek
#define		RC5_T			1778
#define		FAULT_TIME_RC5	RC5_T*17
#define 	TOLERANCE		(RC5_T/10)	//  10%	

#define		FL_RC5_RESET	_BV(0)	//  
#define		FL_STBIT_1		_BV(1)	//   
#define	 	FL_SAMPLE		_BV(7)	//  
#define	 	FL_FAULT		_BV(2)	//  


typedef struct struct_rc5 {
		uint16_t data;
		uint8_t flag;
		uint8_t data_ok;
		uint8_t error;
		} struct_rc5;

struct_rc5 rc5;



void setting_rc5(void){
	//TCCR1B|= _BV(CS10);	// 
	//TCCR1B|= _BV(ICNC1);	//.      
	//TCCR1B&= ~_BV(ICES1);	//       
	// ICESn =0,      ,  ICESn = 1,   . 
	TCCR1B= _BV(CS10)|_BV(ICNC1);

	//   -
	//_BV(TICIE1)      - 1; 
	//_BV(OCIE1A);//    
	TIMSK|=_BV(TICIE1)|_BV(OCIE1A);
	OCR1A=RC5_T;
	//TCNT1=0;
}



ISR(TIMER1_CAPT_vect){//    
static uint16_t period, receive;
static uint8_t counter_bit;
uint16_t capture;

	//capture=ICR1;//   7,2  
	capture=TCNT1;
	
	if ( rc5.flag & FL_SAMPLE ){
		if ( (capture > period-TOLERANCE)&&(capture < period+TOLERANCE) )//   (  capture  )
			{
			TCNT1=0;
			receive<<=1;	
			if ( (TCCR1B & _BV(ICES1))==0 ) receive|=1;//  .      ==1
		
			if (++counter_bit==12) 
				{
				rc5.data=receive;
				rc5.data_ok=1;
				rc5.flag &= ~FL_FAULT;
				}
			}
		TCCR1B ^= _BV(ICES1);//  ICESn = 1,   . ICESn =0,   	//if ( TCCR1B & _BV(ICES1) ) TCCR1B &= ~_BV(ICES1); else TCCR1B|= _BV(ICES1);
		}
	else
	if ( (rc5.flag & FL_STBIT_1)==0 ){// 
		OCR1A=FAULT_TIME_RC5;
		TCNT1=0;
		rc5.flag = FL_STBIT_1|FL_RC5_RESET; 
		}
	else
	if ( rc5.flag & FL_STBIT_1 ){//  
		period=capture;
		if (( period > RC5_T-TOLERANCE )&&( period < RC5_T+TOLERANCE ))
			{
			TCNT1=0;
			rc5.flag = FL_SAMPLE|FL_RC5_RESET|FL_FAULT;
			receive=counter_bit=0;
			TCCR1B |= _BV(ICES1);//ICESn =0,  ,  ICESn = 1,  . 
			}
		}
}

ISR (TIMER1_COMPA_vect){
	if ( rc5.flag & FL_RC5_RESET ){
		if ( rc5.flag & FL_FAULT ) rc5.error=1;
		rc5.flag = 0;
		TCCR1B &= ~_BV(ICES1);//ICESn =0,  ; ICESn = 1,  .
		}
} 

//------------------UART------------------------------------
void USART_Init( void )
{
	UBRRL = 12;// Set baud rate 4800
	UCSRB = (1<<TXEN);//|(1<<RXCIE)|(1<<RXEN);///* Enable transmitter */
	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);/* Set frame format: 8data, 2stop bit */
}

void USART_Transmit( unsigned char data )
{
//	UCSRB|=_BV(TXEN);
//  Wait for empty transmit buffer
	while ( !( UCSRA & (1<<UDRE)) )
	;
	UDR = data;
}

void puts_uart(uint8_t *str){
while (*str) USART_Transmit(*(str++));
}
//------------------UART------------------------------------

int main(void){
uint8_t buffer[20], count=0, count_er=0;
	setting_rc5();
	USART_Init();
	sei();
	
	PORTC=_BV(5);//     (   HyperTerminal)
	
	itoa(RC5_T, buffer, 10);
			puts_uart(buffer);
			USART_Transmit('\r');
			
	
	while(1){
		if ((PINC&_BV(5))==0){
				while((PINC&_BV(5))==0);
					puts_uart("test\r");
				}
		
		if (rc5.data_ok==1){
			rc5.data_ok=0;
			
			puts_uart("\r---");
			itoa(count++, buffer, 10);
			puts_uart(buffer);
			puts_uart("---\r");
			itoa( rc5.data, buffer,2);
			puts_uart(buffer);
			if (rc5.data&_BV(11)) puts_uart("\r  = 1");else puts_uart("\r  = 0");
			puts_uart("\r = ");
			itoa( (rc5.data&0b011111000000)>>6, buffer, 10);
			puts_uart(buffer);
			USART_Transmit(' ');
			puts_uart("\r = ");
			itoa( (rc5.data&0b111111), buffer, 10);
			puts_uart(buffer);
			USART_Transmit('\r');
			}
		if (rc5.error==1){
			rc5.error=0;
			puts_uart("\rerror");
			USART_Transmit(' ');
			utoa(count_er++, buffer, 10);
			puts_uart(buffer);
			USART_Transmit('\r');
			
	
			}

	}

}






